﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win.SupportDialogs.FilterUIProvider;
using Infragistics.Win.UltraWinDataSource;
using Infragistics.Win.UltraWinGrid;
using Infragistics.Win;

namespace WindowsFormsApplication269
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

			// The two columns which we'll populate with dummy data
			ultraDataColumn1 = ultraDataSource1.Band.Columns.Add("1");
			ultraDataColumn2 = ultraDataSource1.Band.Columns.Add("2");

			// The formula column that will be computed from values of column 1 and column 2. We'll be filtering on this column in the repro scenario
			Infragistics.Win.UltraWinGrid.UltraGridColumn ultraGridColumn3 = this.ultraGridBand1.Columns.Add("3", "Formula Column");
			ultraGridColumn3.Formula = "concatenate(if (isnull( [" + ultraDataColumn1.Key + "] ), \"\",[" + ultraDataColumn1.Key + "]), if (isnull( [" + ultraDataColumn2.Key + "] ), \"\",[" + ultraDataColumn2.Key + "]))";
			ultraGridColumn3.Header.VisiblePosition = 2;

			this.ultraGrid1.DataSource = this.ultraDataSource1;
			this.ultraGrid1.DataSource = this.ultraDataSource1;

			this.ultraGrid1.MouseDown += HandleMouseDown;
			this.ultraDataSource1.CellDataRequested += HandleCellDataRequested;

			RefreshGrid();
        }

		// Mouse handling event that really only serves to refresh the datasource in the event that the user clicks on the group header box
		private void HandleMouseDown(object sender, MouseEventArgs e)
		{
			UltraGrid grid = (UltraGrid)sender;
			UIElement element = grid.DisplayLayout.UIElement.LastElementEntered;

			// See if the element is a HeaderUIElement. This will probably never happen
			// because the header element is filled with a TexUIElement, but it's best to 
			// cover all the bases. 
			if (element as GroupByBoxUIElement != null)
			{
				RefreshGrid();
			}
			else
			{
				return;
			}
		}

		// Refresh the rows in the datasource connected to the grid
		private void RefreshGrid()
		{
			ultraDataSource1.SuspendBindingNotifications();

			ultraDataSource1.Rows.SetCount(0);
			ultraDataSource1.Rows.Clear();

			for (int i = 0; i < 10000; i++)
			{
				ultraDataSource1.Rows.Add(false);
			}


			ultraDataSource1.ResumeBindingNotifications();
		}

		// For the non-formula columns, we'll just set them to some aribtrary values after 1000. I'm setting the ones before 1000 all to blank
		// because that's the one I have in mind to reproduce the filtering issue.
		private void HandleCellDataRequested(object sender, CellDataRequestedEventArgs e)
		{
			if (e.Row.Index < 1000)
			{
				e.CacheData = true;
				e.Data = null;
			}
			else if (e.Column.Key.Equals(ultraDataColumn1.Key))
			{
				if (e.Row.Index % 2 - 1 == 0)
				{
					e.CacheData = true;
					e.Data = "this";
				}
				else
				{
					e.CacheData = true;
					e.Data = null;
				}

			}
			else if (e.Column.Key.Equals(ultraDataColumn2.Key))
			{
				if (e.Row.Index % 2 + 1 == 1)
				{
					e.CacheData = true;
					e.Data = "that";
				}
				else
				{
					e.CacheData = true;
					e.Data = null;
				}
			}
		}

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
			Infragistics.Win.UltraWinCalcManager.UltraCalcManager calcMan = new Infragistics.Win.UltraWinCalcManager.UltraCalcManager(this.components);
			calcMan.DeferredCalculationsEnabled = false;
			ultraGrid1.CalcManager = calcMan;
        }

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		/// <param name="disposing">true if managed resources should be disposed; otherwise, false.</param>
		protected override void Dispose(bool disposing)
		{
			this.ultraGrid1.MouseDown -= HandleMouseDown;
			this.ultraDataSource1.CellDataRequested -= HandleCellDataRequested;

			if (disposing && (components != null))
			{
				components.Dispose();
			}
			base.Dispose(disposing);
		}

		
    }
}
